@@ -11,9 +11,12 @@
 #include <osmocom/core/logging.h>
 #include <osmocom/core/application.h>
 #include <osmocom/core/fsm.h>
+#include <osmocom/core/stats.h>
+#include <osmocom/core/rate_ctr.h>
 #include <osmocom/vty/vty.h>
 #include <osmocom/vty/telnet_interface.h>
 #include <osmocom/vty/logging.h>
+#include <osmocom/vty/stats.h>
 #include <osmocom/vty/misc.h>
 
 #include <osmocom/sigtran/osmo_ss7.h>
@@ -28,16 +31,17 @@ static const char *config_file;
 
 static struct osmo_sccp_instance *g_sccp;
 
-static struct osmo_sccp_instance *sua_server_helper(int local_port, const char *local_address, int local_pc,
+static struct osmo_sccp_instance *sua_server_helper(enum osmo_ss7_asp_protocol protocol,
+						    int local_port, const char *local_address, int local_pc,
 						    int remote_port, const char *remote_address, int remote_pc)
 {
 	struct osmo_sccp_instance *sccp;
 
-	sccp = osmo_sccp_simple_server(NULL, local_pc, OSMO_SS7_ASP_PROT_M3UA, local_port, local_address);
+	sccp = osmo_sccp_simple_server(NULL, local_pc, protocol, local_port, local_address);
 	if (sccp == NULL)
 		return NULL;
 
-	osmo_sccp_simple_server_add_clnt(sccp, OSMO_SS7_ASP_PROT_M3UA, "client", remote_pc, local_port,
+	osmo_sccp_simple_server_add_clnt(sccp, protocol, "client", remote_pc, local_port,
 					 remote_port, remote_address);
 
 	return sccp;
@@ -110,6 +114,7 @@ static void usage(void) {
 			"             [-r REMOTE_ADDRESS[:REMOTE_PORT]]\n"
 			"             [-L LOCAL_POINT_CODE] [-R REMOTE_POINT_CODE]\n"
 			"Options:\n"
+			"  -p: protocol to use (m3ua, sua, ipa; default is ipa)\n"
 			"  -c: Run in client mode (default is server mode)\n"
 			"  -l: local IP address and SCTP port (default is %s:%d in server mode,\n"
 			"                                       %s:%d in client mode)\n"
@@ -117,6 +122,7 @@ static void usage(void) {
 			"                                       %s:%d in client mode)\n"
 			"  -L: local point code (default is %d in server mode, %d in client mode)\n"
 			"  -R: remote point code (default is %d in server mode, %d in client mode)\n",
+			"  -d: LOGMASK (libosmocore log mask string, e.g. -d DLINP,1:DLSS7,2)\n"
 			DEFAULT_LOCAL_ADDRESS_SERVER, DEFAULT_LOCAL_PORT_SERVER,
 			DEFAULT_LOCAL_ADDRESS_CLIENT, DEFAULT_LOCAL_PORT_CLIENT,
 			DEFAULT_REMOTE_ADDRESS_SERVER, DEFAULT_REMOTE_PORT_SERVER,
@@ -179,9 +185,19 @@ int main(int argc, char **argv)
 	int remote_port = DEFAULT_REMOTE_PORT_SERVER;
 	int remote_pc = DEFAULT_PC_CLIENT;
 	bool lflag = false, rflag = false, Lflag = false, Rflag = false;
+	enum osmo_ss7_asp_protocol protocol = OSMO_SS7_ASP_PROT_M3UA;
 
-	while ((ch = getopt(argc, argv, "cl:r:L:R:C:")) != -1) {
+	void *tall_ctx = talloc_named_const(NULL, 1, "sccp_demo_user");
+	init_logging(tall_ctx);
+
+	while ((ch = getopt(argc, argv, "p:cl:r:L:R:C:d:")) != -1) {
 		switch (ch) {
+		case 'p':
+			rc = get_string_value(osmo_ss7_asp_protocol_vals, optarg);
+			if (rc < 0)
+				exit(1);
+			protocol = rc;
+			break;
 		case 'c':
 			client = true;
 
@@ -228,6 +244,9 @@ int main(int argc, char **argv)
 		case 'C':
 			config_file = optarg;
 			break;
+		case 'd':
+			log_parse_category_mask(osmo_stderr_target, optarg);
+			break;
 		default:
 			usage();
 		}
@@ -243,14 +262,15 @@ int main(int argc, char **argv)
 	signal(SIGUSR1, &signal_handler);
 	signal(SIGUSR2, &signal_handler);
 
-	void *tall_ctx = talloc_named_const(NULL, 1, "sccp_demo_user");
-	init_logging(tall_ctx);
+	osmo_stats_init(tall_ctx);
+	rate_ctr_init(tall_ctx);
 	OSMO_ASSERT(osmo_ss7_init() == 0);
 	osmo_fsm_log_addr(false);
 	vty_info.tall_ctx = tall_ctx;
 	vty_init(&vty_info);
 	logging_vty_add_cmds();
 	osmo_talloc_vty_add_cmds();
+	osmo_stats_vty_add_cmds();
 	osmo_fsm_vty_add_cmds();
 	osmo_ss7_vty_init_asp(NULL);
 	osmo_sccp_vty_init();
@@ -273,7 +293,7 @@ int main(int argc, char **argv)
 	}
 
 	if (client) {
-		g_sccp = osmo_sccp_simple_client(NULL, "client", local_pc, OSMO_SS7_ASP_PROT_M3UA,
+		g_sccp = osmo_sccp_simple_client(NULL, "client", local_pc, protocol,
 						 local_port, local_address, remote_port, remote_address);
 		if (g_sccp == NULL) {
 			perror("Could not create SCCP client");
@@ -281,7 +301,7 @@ int main(int argc, char **argv)
 		}
 		sccp_test_user_vty_install(g_sccp, OSMO_SCCP_SSN_BSSAP);
 	} else {
-		g_sccp = sua_server_helper(local_port, local_address, local_pc,
+		g_sccp = sua_server_helper(protocol, local_port, local_address, local_pc,
 					   remote_port, remote_address, remote_pc);
 		if (g_sccp == NULL) {
 			perror("Could not create SCCP server");
@@ -289,6 +309,8 @@ int main(int argc, char **argv)
 		}
 		sccp_test_server_init(g_sccp);
 	}
+	g_calling_addr.pc = local_pc;
+	g_called_addr.pc = remote_pc;
 
 	while (1) {
 		osmo_select_main(0);
