@@ -9,6 +9,7 @@
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/prim.h>
 #include <osmocom/core/socket.h>
+#include <osmocom/core/rate_ctr.h>
 
 extern struct llist_head osmo_ss7_instances;
 
@@ -24,6 +25,7 @@ int osmo_ss7_find_free_rctx(struct osmo_ss7_instance *inst);
 bool osmo_ss7_pc_is_local(struct osmo_ss7_instance *inst, uint32_t pc);
 int osmo_ss7_pointcode_parse(struct osmo_ss7_instance *inst, const char *str);
 int osmo_ss7_pointcode_parse_mask_or_len(struct osmo_ss7_instance *inst, const char *in);
+const char *osmo_ss7_pointcode_print_buf(char *buf, size_t buf_len, const struct osmo_ss7_instance *inst, uint32_t pc);
 const char *osmo_ss7_pointcode_print(const struct osmo_ss7_instance *inst, uint32_t pc);
 const char *osmo_ss7_pointcode_print2(const struct osmo_ss7_instance *inst, uint32_t pc);
 
@@ -113,6 +115,7 @@ int osmo_ss7_instance_set_pc_fmt(struct osmo_ss7_instance *inst,
 struct osmo_sccp_instance *osmo_ss7_ensure_sccp(struct osmo_ss7_instance *inst);
 
 uint8_t osmo_ss7_pc_width(const struct osmo_ss7_pc_fmt *pc_fmt);
+uint32_t osmo_ss7_pc_normalize(const struct osmo_ss7_pc_fmt *pc_fmt, uint32_t pc);
 
 /***********************************************************************
  * MTP Users (Users of MTP, such as SCCP or ISUP)
@@ -308,6 +311,9 @@ struct osmo_ss7_as {
 	/*! Were we allocated by "simple client" support? */
 	bool simple_client_allocated;
 
+	/*! Rate Counter Group */
+	struct rate_ctr_group *ctrg;
+
 	struct {
 		char *name;
 		char *description;
@@ -344,8 +350,8 @@ osmo_ss7_as_find_or_create(struct osmo_ss7_instance *inst, const char *name,
 int osmo_ss7_as_add_asp(struct osmo_ss7_as *as, const char *asp_name);
 int osmo_ss7_as_del_asp(struct osmo_ss7_as *as, const char *asp_name);
 void osmo_ss7_as_destroy(struct osmo_ss7_as *as);
-bool osmo_ss7_as_has_asp(struct osmo_ss7_as *as,
-			 struct osmo_ss7_asp *asp);
+bool osmo_ss7_as_has_asp(const struct osmo_ss7_as *as,
+			 const struct osmo_ss7_asp *asp);
 bool osmo_ss7_as_active(const struct osmo_ss7_as *as);
 bool osmo_ss7_as_tmode_compatible_xua(struct osmo_ss7_as *as, uint32_t m3ua_tmt);
 void osmo_ss7_asp_disconnect(struct osmo_ss7_asp *asp);
@@ -410,6 +416,9 @@ struct osmo_ss7_asp {
 	/*! Were we allocated by "simple client" support? */
 	bool simple_client_allocated;
 
+	/*! Rate Counter Group */
+	struct rate_ctr_group *ctrg;
+
 	/*! Pending message for non-blocking IPA read */
 	struct msgb *pending_msg;
 
@@ -425,9 +434,17 @@ struct osmo_ss7_asp {
 		struct osmo_ss7_asp_peer local;
 		struct osmo_ss7_asp_peer remote;
 		uint8_t qos_class;
+		uint32_t quirks;
 	} cfg;
 };
 
+/*! Peer SG doesn't send NTFY(AS-INACTIVE) after ASP-UP procedure */
+#define OSMO_SS7_ASP_QUIRK_NO_NOTIFY		0x00000001
+/*! Accept DAUD in ASP role (RFC states only permitted in ASP->SG role) */
+#define OSMO_SS7_ASP_QUIRK_DAUD_IN_ASP		0x00000002
+/*! Accept SSNM even if ASP is in AS-INACTIVE state */
+#define OSMO_SS7_ASP_QUIRK_SNM_INACTIVE		0x00000004
+
 int osmo_ss7_asp_peer_snprintf(char* buf, size_t buf_len, struct osmo_ss7_asp_peer *peer);
 int osmo_ss7_asp_peer_set_hosts(struct osmo_ss7_asp_peer *peer, void *talloc_ctx,
 				const char* const* hosts, size_t host_cnt);
@@ -451,6 +468,7 @@ int osmo_ss7_asp_send(struct osmo_ss7_asp *asp, struct msgb *msg);
 int osmo_ss7_asp_restart(struct osmo_ss7_asp *asp);
 int osmo_ss7_asp_use_default_lm(struct osmo_ss7_asp *asp, int log_level);
 bool osmo_ss7_asp_active(const struct osmo_ss7_asp *asp);
+int osmo_ss7_asp_get_log_subsys(const struct osmo_ss7_asp *asp);
 
 /*! Weak function to handle payload for unknown/unsupported PPID or IPA StreamID.
  *  This function can be overridden by application code to implement whatever handling
