@@ -51,6 +51,7 @@ enum osmo_scu_prim_type {
 #define OSMO_SCCP_ADDR_T_SSN	0x0004 /* subsystem number */
 #define OSMO_SCCP_ADDR_T_IPv4	0x0008
 #define OSMO_SCCP_ADDR_T_IPv6	0x0010
+#define OSMO_SCCP_ADDR_T_MASK	0x001f
 
 /* Q.713 3.4.1 + RFC 3868 3.10.2.3 */
 enum osmo_sccp_routing_ind {
@@ -131,11 +132,29 @@ enum osmo_sccp_ssn {
 	OSMO_SCCP_SSN_PCAP		= 249,
 	OSMO_SCCP_SSN_BSC_BSSAP_LE	= 250,
 	OSMO_SCCP_SSN_MSC_BSSAP_LE	= 251,
-	OSMO_SCCP_SSN_SMLC_BSSAP	= 252,
+	OSMO_SCCP_SSN_SMLC_BSSAP_LE	= 252,
 	OSMO_SCCP_SSN_BSS_OAM		= 253,
 	OSMO_SCCP_SSN_BSSAP		= 254,
 };
 
+/* Q.711 6.3.2.2.5 Signalling point status */
+enum osmo_sccp_sp_status {
+	OSMO_SCCP_SP_S_INACCESSIBLE	= 1,
+	OSMO_SCCP_SP_S_CONGESTED	= 2,
+	OSMO_SCCP_SP_S_ACCESSIBLE	= 3,
+};
+/* Q.711 6.3.2.2.6 Remote SCCP status */
+enum osmo_sccp_rem_sccp_status {
+	OSMO_SCCP_REM_SCCP_S_AVAILABLE 			= 1,
+	OSMO_SCCP_REM_SCCP_S_UNAVAILABLE_UNKNOWN	= 2,
+	OSMO_SCCP_REM_SCCP_S_UNEQUIPPED			= 3,
+	OSMO_SCCP_REM_SCCP_S_INACCESSIBLE		= 4,
+	OSMO_SCCP_REM_SCCP_S_CONGESTED			= 5,
+};
+
+/* legacy shim for name change */
+#define OSMO_SCCP_SSN_SMLC_BSSAP OSMO_SCCP_SSN_SMLC_BSSAP_LE
+
 extern const struct value_string osmo_sccp_ssn_names[];
 static inline const char *osmo_sccp_ssn_name(enum osmo_sccp_ssn val)
 { return get_value_string(osmo_sccp_ssn_names, val); }
@@ -222,6 +241,22 @@ struct osmo_scu_notice_param {
 	/* user data */
 };
 
+/* OSMO_SCU_PRIM_N_STATE */
+struct osmo_scu_state_param {
+	uint32_t affected_pc;
+	uint32_t affected_ssn;
+	bool user_in_service; /* true: UIS; false: UOS */
+	uint32_t ssn_multiplicity_ind;
+};
+
+/* OSMO_ASCU_PRIM_N_PCSTATE */
+struct osmo_scu_pcstate_param {
+	uint32_t affected_pc;
+	uint32_t restricted_importance_level;
+	enum osmo_sccp_sp_status sp_status;
+	enum osmo_sccp_rem_sccp_status remote_sccp_status;
+};
+
 struct osmo_scu_prim {
 	struct osmo_prim_hdr oph;
 	union {
@@ -231,6 +266,8 @@ struct osmo_scu_prim {
 		struct osmo_scu_reset_param reset;
 		struct osmo_scu_unitdata_param unitdata;
 		struct osmo_scu_notice_param notice;
+		struct osmo_scu_state_param state;
+		struct osmo_scu_pcstate_param pcstate;
 	} u;
 };
 
@@ -244,8 +281,7 @@ struct osmo_sccp_user;
 
 void osmo_sccp_vty_init(void);
 
-struct osmo_sccp_instance *
-osmo_sccp_instance_create(struct osmo_ss7_instance *ss7, void *priv);
+struct osmo_sccp_instance *osmo_sccp_instance_create(struct osmo_ss7_instance *ss7, void *priv);
 void osmo_sccp_instance_destroy(struct osmo_sccp_instance *inst);
 struct osmo_ss7_instance *osmo_sccp_get_ss7(const struct osmo_sccp_instance *sccp);
 struct osmo_sccp_instance *osmo_sccp_get_sccp(const struct osmo_sccp_user *scu);
@@ -254,22 +290,19 @@ void osmo_sccp_user_unbind(struct osmo_sccp_user *scu);
 void osmo_sccp_user_set_priv(struct osmo_sccp_user *scu, void *priv);
 void *osmo_sccp_user_get_priv(struct osmo_sccp_user *scu);
 
-struct osmo_sccp_user *
-osmo_sccp_user_bind_pc(struct osmo_sccp_instance *inst, const char *name,
-		       osmo_prim_cb prim_cb, uint16_t ssn, uint32_t pc);
+struct osmo_sccp_user *osmo_sccp_user_bind_pc(struct osmo_sccp_instance *inst, const char *name,
+					      osmo_prim_cb prim_cb, uint16_t ssn, uint32_t pc);
 
-struct osmo_sccp_user *
-osmo_sccp_user_bind(struct osmo_sccp_instance *inst, const char *name,
-		    osmo_prim_cb prim_cb, uint16_t ssn);
-struct osmo_sccp_user *
-osmo_sccp_user_find(struct osmo_sccp_instance *inst, uint16_t ssn, uint32_t pc);
+struct osmo_sccp_user *osmo_sccp_user_bind(struct osmo_sccp_instance *inst, const char *name,
+					   osmo_prim_cb prim_cb, uint16_t ssn);
+struct osmo_sccp_user *osmo_sccp_user_find(struct osmo_sccp_instance *inst, uint16_t ssn, uint32_t pc);
 
 int osmo_sccp_user_sap_down(struct osmo_sccp_user *scu, struct osmo_prim_hdr *oph);
 int osmo_sccp_user_sap_down_nofree(struct osmo_sccp_user *scu, struct osmo_prim_hdr *oph);
 
-struct osmo_ss7_instance *
-osmo_sccp_addr_by_name(struct osmo_sccp_addr *dest_addr,
-		       const char *name);
+struct osmo_ss7_instance *osmo_sccp_addr_by_name(struct osmo_sccp_addr *dest_addr, const char *name);
+int osmo_sccp_addr_by_name_local(struct osmo_sccp_addr *dest_addr, const char *name,
+				 const struct osmo_ss7_instance *inst);
 
 const char *osmo_sccp_name_by_addr(const struct osmo_sccp_addr *addr);
 
