@@ -44,6 +44,7 @@
 #include "xua_as_fsm.h"
 #include "xua_asp_fsm.h"
 #include "xua_internal.h"
+#include "ss7_internal.h"
 
 #define M3UA_MSGB_SIZE 1500
 
@@ -448,7 +449,7 @@ int m3ua_decode_notify(struct osmo_xlm_prim_notify *npar, void *ctx,
 }
 
 /***********************************************************************
- * Transmitting M3UA messsages to SCTP
+ * Transmitting M3UA messages to SCTP
  ***********************************************************************/
 
 /* Convert M3UA from xua_msg to msgb and set PPID/stream */
@@ -511,7 +512,7 @@ int m3ua_tx_xua_as(struct osmo_ss7_as *as, struct xua_msg *xua)
 }
 
 /***********************************************************************
- * Receiving M3UA messsages from SCTP
+ * Receiving M3UA messages from SCTP
  ***********************************************************************/
 
 /* obtain the destination point code from a M3UA message in XUA fmt * */
@@ -532,28 +533,12 @@ struct m3ua_data_hdr *data_hdr_from_m3ua(struct xua_msg *xua)
 	return data_hdr;
 }
 
-/* if given ASP only has one AS, return that AS */
-static struct osmo_ss7_as *find_single_as_for_asp(struct osmo_ss7_asp *asp)
-{
-	struct osmo_ss7_as *as, *as_found = NULL;
-
-	llist_for_each_entry(as, &asp->inst->as_list, list) {
-		if (!osmo_ss7_as_has_asp(as, asp))
-			continue;
-		/* check if we already had found another AS within this ASP -> not unique */
-		if (as_found)
-			return NULL;
-		as_found = as;
-	}
-
-	return as_found;
-}
-
 static int m3ua_rx_xfer(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 {
 	struct xua_msg_part *rctx_ie = xua_msg_find_tag(xua, M3UA_IEI_ROUTE_CTX);
 	struct m3ua_data_hdr *dh;
 	struct osmo_ss7_as *as;
+	int rc;
 
 	LOGPASP(asp, DLM3UA, LOGL_DEBUG, "m3ua_rx_xfer\n");
 
@@ -565,34 +550,11 @@ static int m3ua_rx_xfer(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 		return M3UA_ERR_UNSUPP_MSG_TYPE;
 	}
 
-	if (rctx_ie) {
-		uint32_t rctx = xua_msg_part_get_u32(rctx_ie);
-		/* Use routing context IE to look up the AS for which the
-		 * message was received. */
-		as = osmo_ss7_as_find_by_rctx(asp->inst, rctx);
-		if (!as) {
-			LOGPASP(asp, DLM3UA, LOGL_ERROR, "%s(): invalid routing context: %u\n",
-				__func__, rctx);
-			return M3UA_ERR_INVAL_ROUT_CTX;
-		}
+	rc = xua_find_as_for_asp(&as, asp, rctx_ie);
+	if (rc)
+		return rc;
 
-		/* Verify that this ASP is part of the AS. */
-		if (!osmo_ss7_as_has_asp(as, asp)) {
-			LOGPASP(asp, DLM3UA, LOGL_ERROR,
-				"%s(): This Application Server Process is not part of the AS %s "
-				"resolved by routing context %u\n", __func__, (as)->cfg.name, rctx);
-			return M3UA_ERR_NO_CONFGD_AS_FOR_ASP;
-		}
-	} else {
-		/* no explicit routing context; this only works if there is only one AS in the ASP */
-		as = find_single_as_for_asp(asp);
-		if (!as) {
-			LOGPASP(asp, DLM3UA, LOGL_ERROR,
-				"%s(): ASP sent M3UA without Routing Context IE but unable to uniquely "
-				"identify the AS for this message\n", __func__);
-			return M3UA_ERR_INVAL_ROUT_CTX;
-		}
-	}
+	rate_ctr_inc2(as->ctrg, SS7_AS_CTR_RX_MSU_TOTAL);
 
 	/* FIXME: check for AS state == ACTIVE */
 
@@ -718,6 +680,8 @@ static int m3ua_rx_asp(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 	return 0;
 }
 
+static int m3ua_rx_snm(struct osmo_ss7_asp *asp, struct xua_msg *xua);
+
 /*! \brief process M3UA message received from socket
  *  \param[in] asp Application Server Process receiving \ref msg
  *  \param[in] msg received message buffer
@@ -730,7 +694,7 @@ int m3ua_rx_msg(struct osmo_ss7_asp *asp, struct msgb *msg)
 	OSMO_ASSERT(asp->cfg.proto == OSMO_SS7_ASP_PROT_M3UA);
 
 	/* caller owns msg memory, we shall neither free it here nor
-	 * keep references beyond the executin of this function and its
+	 * keep references beyond the execution of this function and its
 	 * callees */
 
 	xua = xua_from_msg(M3UA_VERSION, msgb_length(msg), msgb_data(msg));
@@ -778,10 +742,7 @@ int m3ua_rx_msg(struct osmo_ss7_asp *asp, struct msgb *msg)
 		rc = m3ua_rx_rkm(asp, xua);
 		break;
 	case M3UA_MSGC_SNM:
-		/* FIXME */
-		LOGPASP(asp, DLM3UA, LOGL_NOTICE, "Received unsupported M3UA "
-			"Message Class %u\n", xua->hdr.msg_class);
-		err = m3ua_gen_error_msg(M3UA_ERR_UNSUPP_MSG_CLASS, msg);
+		rc = m3ua_rx_snm(asp, xua);
 		break;
 	default:
 		LOGPASP(asp, DLM3UA, LOGL_NOTICE, "Received unknown M3UA "
@@ -794,10 +755,249 @@ int m3ua_rx_msg(struct osmo_ss7_asp *asp, struct msgb *msg)
 		err = m3ua_gen_error_msg(rc, msg);
 
 out:
-	if (err)
+	if (err) {
 		m3ua_tx_xua_asp(asp, err);
+		xua_msg_free(err);
+	}
 
 	xua_msg_free(xua);
 
 	return rc;
 }
+
+/***********************************************************************
+ * SSNM msg generation
+ ***********************************************************************/
+
+/* 3.4.1 Destination Unavailable (DUNA) */
+static struct xua_msg *m3ua_encode_duna(const uint32_t *rctx, unsigned int num_rctx,
+					const uint32_t *aff_pc, unsigned int num_aff_pc,
+					const char *info_string)
+{
+	struct xua_msg *xua = xua_msg_alloc();
+
+	xua->hdr = XUA_HDR(M3UA_MSGC_SNM, M3UA_SNM_DUNA);
+	xua->hdr.version = M3UA_VERSION;
+	if (rctx)
+		xua_msg_add_data(xua, M3UA_IEI_ROUTE_CTX, num_rctx * sizeof(*rctx), (const uint8_t *)rctx);
+
+	xua_msg_add_data(xua, M3UA_IEI_AFFECTED_PC, num_aff_pc * sizeof(*aff_pc), (const uint8_t *) aff_pc);
+
+	if (info_string) {
+		xua_msg_add_data(xua, M3UA_IEI_INFO_STRING,
+				 strlen(info_string)+1,
+				 (const uint8_t *) info_string);
+	}
+	return xua;
+}
+
+/* 3.4.2 Destination Available (DAVA) */
+static struct xua_msg *m3ua_encode_dava(const uint32_t *rctx, unsigned int num_rctx,
+					const uint32_t *aff_pc, unsigned int num_aff_pc,
+					const char *info_string)
+{
+	/* encoding is exactly identical to DUNA */
+	struct xua_msg *xua = m3ua_encode_duna(rctx, num_rctx, aff_pc, num_aff_pc, info_string);
+	if (xua)
+		xua->hdr.msg_type = M3UA_SNM_DAVA;
+	return xua;
+}
+
+#if 0 /* not used so far */
+/* 3.4.3 Destination Available (DAUD) */
+static struct xua_msg *m3ua_encode_daud(const uint32_t *rctx, unsigned int num_rctx,
+					const uint32_t *aff_pc, unsigned int num_aff_pc,
+					const char *info_string)
+{
+	/* encoding is exactly identical to DUNA */
+	struct xua_msg *xua = m3ua_encode_duna(rctx, num_rctx, aff_pc, num_aff_pc, info_string);
+	if (xua)
+		xua->hdr.msg_type = M3UA_SNM_DAUD;
+	return xua;
+}
+#endif
+
+/* 3.4.5 Destination User Part Unavailable (DUPU) */
+static struct xua_msg *m3ua_encode_dupu(const uint32_t *rctx, unsigned int num_rctx,
+					uint32_t dpc, uint16_t user, uint16_t cause,
+					const char *info_string)
+{
+	struct xua_msg *xua = xua_msg_alloc();
+	uint32_t user_cause = (cause << 16) | user;
+
+	xua->hdr = XUA_HDR(M3UA_MSGC_SNM, M3UA_SNM_DUPU);
+	xua->hdr.version = M3UA_VERSION;
+	if (rctx)
+		xua_msg_add_data(xua, M3UA_IEI_ROUTE_CTX, num_rctx * sizeof(*rctx), (const uint8_t *)rctx);
+
+	xua_msg_add_u32(xua, M3UA_IEI_AFFECTED_PC, dpc);
+	xua_msg_add_u32(xua, M3UA_IEI_USER_CAUSE, user_cause);
+
+	if (info_string) {
+		xua_msg_add_data(xua, M3UA_IEI_INFO_STRING,
+				 strlen(info_string)+1,
+				 (const uint8_t *) info_string);
+	}
+	return xua;
+}
+
+/*! Transmit SSNM DUNA/DAVA message indicating [un]availability of certain point code[s]
+ *  \param[in] asp ASP through which to transmit message. Must be ACTIVE.
+ *  \param[in] rctx array of Routing Contexts in network byte order.
+ *  \param[in] num_rctx number of rctx
+ *  \param[in] aff_pc array of 'Affected Point Code' in network byte order.
+ *  \param[in] num_aff_pc number of aff_pc
+ *  \param[in] info_string optional information string (can be NULL).
+ *  \param[in] available are aff_pc now available (true) or unavailable (false) */
+void m3ua_tx_snm_available(struct osmo_ss7_asp *asp, const uint32_t *rctx, unsigned int num_rctx,
+			   const uint32_t *aff_pc, unsigned int num_aff_pc,
+			   const char *info_string, bool available)
+{
+	struct xua_msg *xua;
+
+	if (available)
+		xua = m3ua_encode_dava(rctx, num_rctx, aff_pc, num_aff_pc, info_string);
+	else
+		xua = m3ua_encode_duna(rctx, num_rctx, aff_pc, num_aff_pc, info_string);
+
+	m3ua_tx_xua_asp(asp, xua);
+	xua_msg_free(xua);
+}
+
+/*! Transmit SSNM SCON message indicating congestion
+ *  \param[in] asp ASP through which to transmit message. Must be ACTIVE.
+ *  \param[in] rctx array of Routing Contexts in network byte order.
+ *  \param[in] num_rctx number of rctx
+ *  \param[in] aff_pc array of 'Affected Point Code' in network byte order.
+ *  \param[in] num_aff_pc number of aff_pc
+ *  \param[in] concerned_dpc optional concerned DPC (can be NULL)
+ *  \param[in] cong_level optional congestion level (can be NULL)
+ *  \param[in] info_string optional information string (can be NULL). */
+void m3ua_tx_snm_congestion(struct osmo_ss7_asp *asp, const uint32_t *rctx, unsigned int num_rctx,
+			    const uint32_t *aff_pc, unsigned int num_aff_pc,
+			    const uint32_t *concerned_dpc, const uint8_t *cong_level,
+			    const char *info_string)
+{
+	struct xua_msg *xua = xua_msg_alloc();
+
+	xua->hdr = XUA_HDR(M3UA_MSGC_SNM, M3UA_SNM_SCON);
+	xua->hdr.version = M3UA_VERSION;
+	if (rctx)
+		xua_msg_add_data(xua, M3UA_IEI_ROUTE_CTX, num_rctx * sizeof(*rctx), (const uint8_t *)rctx);
+
+	xua_msg_add_data(xua, M3UA_IEI_AFFECTED_PC, num_aff_pc * sizeof(*aff_pc), (const uint8_t *) aff_pc);
+
+	if (concerned_dpc)
+		xua_msg_add_u32(xua, M3UA_IEI_CONC_DEST, *concerned_dpc & 0xffffff);
+	if (cong_level)
+		xua_msg_add_u32(xua, M3UA_IEI_CONG_IND, *cong_level & 0xff);
+	if (info_string)
+		xua_msg_add_data(xua, M3UA_IEI_INFO_STRING, strlen(info_string)+1, (const uint8_t *) info_string);
+
+	m3ua_tx_xua_asp(asp, xua);
+	xua_msg_free(xua);
+}
+
+/*! Transmit SSNM DUPU message indicating user unavailability.
+ *  \param[in] asp ASP through which to transmit message. Must be ACTIVE.
+ *  \param[in] rctx array of Routing Contexts in network byte order.
+ *  \param[in] num_rctx number of rctx
+ *  \param[in] dpc affected point code
+ *  \param[in] user the user (SI) that is unavailable
+ *  \param[in] cause the cause of the user unavailability
+ *  \param[in] info_string optional information string (can be NULL). */
+void m3ua_tx_dupu(struct osmo_ss7_asp *asp, const uint32_t *rctx, unsigned int num_rctx,
+		  uint32_t dpc, uint16_t user, uint16_t cause, const char *info_str)
+{
+	struct xua_msg *xua = m3ua_encode_dupu(rctx, num_rctx, dpc, user, cause, info_str);
+	m3ua_tx_xua_asp(asp, xua);
+	xua_msg_free(xua);
+}
+
+/* received SNM message on ASP side */
+static int m3ua_rx_snm_asp(struct osmo_ss7_asp *asp, struct xua_msg *xua)
+{
+	struct osmo_ss7_as *as = NULL;
+	struct xua_msg_part *rctx_ie = xua_msg_find_tag(xua, M3UA_IEI_ROUTE_CTX);
+	int rc;
+
+	rc = xua_find_as_for_asp(&as, asp, rctx_ie);
+	if (rc)
+		return rc;
+
+	/* report those up the stack so both other ASPs and local SCCP users can be notified */
+	switch (xua->hdr.msg_type) {
+	case M3UA_SNM_DUNA:
+		xua_snm_rx_duna(asp, as, xua);
+		break;
+	case M3UA_SNM_DAVA:
+		xua_snm_rx_dava(asp, as, xua);
+		break;
+	case M3UA_SNM_DUPU:
+		xua_snm_rx_dupu(asp, as, xua);
+		break;
+	case M3UA_SNM_SCON:
+		xua_snm_rx_scon(asp, as, xua);
+		break;
+	case M3UA_SNM_DRST:
+		LOGPASP(asp, DLM3UA, LOGL_NOTICE, "Received unsupported M3UA SNM message type %u\n",
+			xua->hdr.msg_type);
+		/* silently ignore those to not confuse the sender */
+		break;
+	case M3UA_SNM_DAUD:
+		/* RFC states only permitted in ASP->SG direction, not reverse. But some
+		 * equipment still sends it to us as ASP ?!? */
+		if (asp->cfg.quirks & OSMO_SS7_ASP_QUIRK_DAUD_IN_ASP) {
+			LOGPASP(asp, DLM3UA, LOGL_NOTICE, "quirk daud_in_asp active: Accepting DAUD "
+				"despite being in ASP role\n");
+			xua_snm_rx_daud(asp, xua);
+		} else {
+			LOGPASP(asp, DLM3UA, LOGL_ERROR, "DAUD not permitted in ASP role\n");
+			return M3UA_ERR_UNSUPP_MSG_TYPE;
+		}
+		break;
+	default:
+		return M3UA_ERR_UNSUPP_MSG_TYPE;
+	}
+
+	return 0;
+}
+
+/* received SNM message on SG side */
+static int m3ua_rx_snm_sg(struct osmo_ss7_asp *asp, struct xua_msg *xua)
+{
+	switch (xua->hdr.msg_type) {
+	case M3UA_SNM_DAUD:	/* Audit: ASP inquires about availability of Point Codes */
+		xua_snm_rx_daud(asp, xua);
+		break;
+	default:
+		return M3UA_ERR_UNSUPP_MSG_TYPE;
+	}
+
+	return 0;
+}
+
+static int m3ua_rx_snm(struct osmo_ss7_asp *asp, struct xua_msg *xua)
+{
+	/* SNM only permitted in ACTIVE state */
+	if (asp->fi->state != XUA_ASP_S_ACTIVE) {
+		if (asp->fi->state == XUA_ASP_S_INACTIVE &&
+		    asp->cfg.quirks & OSMO_SS7_ASP_QUIRK_SNM_INACTIVE) {
+			LOGPASP(asp, DLM3UA, LOGL_NOTICE, "quirk snm_inactive active: "
+				"Accepting SNM in state %s\n", osmo_fsm_inst_state_name(asp->fi));
+		} else {
+			LOGPASP(asp, DLM3UA, LOGL_ERROR, "Rx M3UA SNM not permitted "
+				"while ASP in state %s\n", osmo_fsm_inst_state_name(asp->fi));
+			return M3UA_ERR_UNEXPECTED_MSG;
+		}
+	}
+
+	switch (asp->cfg.role) {
+	case OSMO_SS7_ASP_ROLE_SG:
+		return m3ua_rx_snm_sg(asp, xua);
+	case OSMO_SS7_ASP_ROLE_ASP:
+		return m3ua_rx_snm_asp(asp, xua);
+	default:
+		return M3UA_ERR_UNSUPP_MSG_CLASS;
+	}
+}
