@@ -3,6 +3,8 @@
 /* (C) 2017 by Harald Welte <laforge@gnumonks.org>
  * All Rights Reserved
  *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
  * based on my 2011 Erlang implementation osmo_ss7/src/sua_sccp_conv.erl
  *
  * References: ITU-T Q.713 and IETF RFC 3868
@@ -25,7 +27,7 @@
 #include <string.h>
 #include <errno.h>
 
-#include <sccp/sccp.h>
+#include <osmocom/sccp/sccp.h>
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/logging.h>
 #include <osmocom/sigtran/sccp_sap.h>
@@ -107,7 +109,7 @@ int osmo_isup_party_encode(struct msgb *msg, const char *in_digits)
 	return num_octets;
 }
 
-/*! \brief Parse wire-encoded SCCP address into omso_sccp_addr
+/*! \brief Parse wire-encoded SCCP address into osmo_sccp_addr
  *  \param[out] out user-allocated output data structure
  *  \param[in] addr wire-encoded SCCP address
  *  \param[in] addrlen Size of \ref addr in bytes
@@ -163,13 +165,15 @@ int osmo_sccp_addr_parse(struct osmo_sccp_addr *out,
 		out->gt.gti = OSMO_SCCP_GTI_TT_ONLY;
 		out->gt.tt = *cur++;
 		/* abort, for national use only */
+		LOGP(DLSUA, LOGL_ERROR, "Unsupported national GTI %u\n", sca->global_title_indicator);
 		return -EINVAL;
 	case SCCP_TITLE_IND_TRANS_NUM_ENC:
 		out->presence |= OSMO_SCCP_ADDR_T_GT;
 		out->gt.gti = OSMO_SCCP_GTI_TT_NPL_ENC;
 		out->gt.tt = *cur++;
 		out->gt.npi = *cur >> 4;
-		switch (*cur++ & 0xF) {
+		encoding = *cur++ & 0xF;
+		switch (encoding) {
 		case 1:
 			odd = true;
 			break;
@@ -177,6 +181,7 @@ int osmo_sccp_addr_parse(struct osmo_sccp_addr *out,
 			odd = false;
 			break;
 		default:
+			LOGP(DLSUA, LOGL_ERROR, "Unknown GT encoding 0x%x\n", encoding);
 			return -1;
 		}
 		break;
@@ -237,11 +242,20 @@ int osmo_sccp_addr_encode(struct msgb *msg, const struct osmo_sccp_addr *in)
 
 	if (in->presence & OSMO_SCCP_ADDR_T_PC) {
 		sca->point_code_indicator = 1;
-		msgb_put_u16le(msg, in->pc & 0x3ff);
+		/* ITU-T Q.713 states that signalling point codes are 14bit */
+		if (in->pc > 0x3fff) {
+			LOGP(DLSUA, LOGL_ERROR, "Invalid Point Code %u requested\n", in->pc);
+			return -EINVAL;
+		}
+		msgb_put_u16le(msg, in->pc & 0x3fff);
 	}
 
 	if (in->presence & OSMO_SCCP_ADDR_T_SSN) {
 		sca->ssn_indicator = 1;
+		if (in->ssn > 0xff) {
+			LOGP(DLSUA, LOGL_ERROR, "Invalid SSN %u requested\n", in->ssn);
+			return -EINVAL;
+		}
 		msgb_put_u8(msg, in->ssn);
 	}
 
@@ -250,6 +264,16 @@ int osmo_sccp_addr_encode(struct msgb *msg, const struct osmo_sccp_addr *in)
 		goto out;
 	}
 
+	if (in->gt.npi && (in->gt.npi > 0xF)) {
+		LOGP(DLSUA, LOGL_ERROR, "Unsupported Numbering Plan %u", in->gt.npi);
+		return -EINVAL;
+	}
+
+	if (in->gt.nai && (in->gt.nai > 0x7F)) {
+		LOGP(DLSUA, LOGL_ERROR, "Unsupported Nature of Address %u", in->gt.nai);
+		return -EINVAL;
+	}
+
 	odd = strlen(in->gt.digits) & 1;
 	switch (in->gt.gti) {
 	case OSMO_SCCP_GTI_NO_GT:
@@ -277,6 +301,9 @@ int osmo_sccp_addr_encode(struct msgb *msg, const struct osmo_sccp_addr *in)
 		msgb_put_u8(msg, (in->gt.npi << 4) | (odd ? 1 : 2));
 		msgb_put_u8(msg, in->gt.nai & 0x7f);
 		break;
+	default:
+		LOGP(DLSUA, LOGL_ERROR, "Unsupported GTI %u requested\n", in->gt.gti);
+		return -EINVAL;
 	}
 	osmo_isup_party_encode(msg, in->gt.digits);
 
@@ -303,7 +330,7 @@ static int sccp_addr_to_sua(struct xua_msg *xua, uint16_t iei, const uint8_t *ad
 	if (rc < 0)
 		return rc;
 
-	LOGP(DLSUA, LOGL_DEBUG, "Parsed Addr: %s\n", osmo_sccp_addr_dump(&osa));
+	LOGP(DLSUA, LOGL_DEBUG, "IEI %u: Parsed Addr: %s\n", iei, osmo_sccp_addr_dump(&osa));
 
 	/* Then re-encode it as SUA address */
 	return xua_msg_add_sccp_addr(xua, iei, &osa);
@@ -678,41 +705,54 @@ static struct xua_msg *sccp_to_xua_opt(struct msgb *msg, uint8_t *ptr_opt, struc
 	/* some bounds checking */
 	if (ptr_opt < msg->data || ptr_opt > msg->tail)
 		return NULL;
+
+	/* Q.713 section 2.3 "Coding of pointers": pointer value all zeros used
+	  to indicate that no optional param is present. */
+	if (*ptr_opt == 0)
+		return xua;
+
 	opt_start = ptr_opt + *ptr_opt;
 	if (opt_start > msg->tail)
 		return NULL;
 
 	oneopt = opt_start;
 
+	enum sccp_parameter_name_codes opt_type = 0; /* dummy value not used */
 	while (oneopt < msg->tail) {
-		uint8_t opt_type = oneopt[0];
+		uint8_t opt_len;
+		uint16_t opt_len16;
+		opt_type = oneopt[0];
 
-		if (opt_type == SCCP_PNC_END_OF_OPTIONAL)
+		switch (opt_type) {
+		case SCCP_PNC_END_OF_OPTIONAL:
 			return xua;
-
-		if (opt_type == SCCP_PNC_LONG_DATA) {
-			uint16_t opt_len16;
+		case SCCP_PNC_LONG_DATA:
 			/* two byte length field */
 			if (oneopt + 2 > msg->tail)
-				return NULL;
+				goto malformed;
 			opt_len16 = oneopt[1] << 8 | oneopt[2];
 			if (oneopt + 3 + opt_len16 > msg->tail)
-				return NULL;
+				goto malformed;
 			xua_msg_add_sccp_opt(xua, opt_type, opt_len16, oneopt+3);
 			oneopt += 3 + opt_len16;
-		} else {
-			uint8_t opt_len;
+			break;
+		default:
 			/* one byte length field */
 			if (oneopt + 1 > msg->tail)
-				return NULL;
+				goto malformed;
 
 			opt_len = oneopt[1];
 			if (oneopt + 2 + opt_len > msg->tail)
-				return NULL;
+				goto malformed;
 			xua_msg_add_sccp_opt(xua, opt_type, opt_len, oneopt+2);
 			oneopt += 2 + opt_len;
 		}
 	}
+	LOGP(DLSUA, LOGL_ERROR, "Parameter %s not found\n", osmo_sccp_pnc_name(SCCP_PNC_END_OF_OPTIONAL));
+	return NULL;
+
+malformed:
+	LOGP(DLSUA, LOGL_ERROR, "Malformed parameter %s (%d)\n", osmo_sccp_pnc_name(opt_type), opt_type);
 	return NULL;
 }
 
@@ -971,6 +1011,7 @@ static void store_local_ref(struct sccp_source_reference *local_ref, struct xua_
 	local_ref->octet3 = tmp32 & 0xff;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_cr(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_request *req = (struct sccp_connection_request *)msg->l2h;
@@ -986,6 +1027,7 @@ static struct xua_msg *sccp_to_xua_cr(struct msgb *msg, struct xua_msg *xua)
 	return sccp_to_xua_opt(msg, &req->optional_start, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static int sua_to_sccp_cr(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_request *req;
@@ -1002,6 +1044,7 @@ static int sua_to_sccp_cr(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &req->optional_start, req->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_cc(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_confirm *cnf = (struct sccp_connection_confirm *)msg->l2h;
@@ -1014,6 +1057,7 @@ static struct xua_msg *sccp_to_xua_cc(struct msgb *msg, struct xua_msg *xua)
 	return sccp_to_xua_opt(msg, &cnf->optional_start, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static int sua_to_sccp_cc(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_confirm *cnf;
@@ -1028,6 +1072,7 @@ static int sua_to_sccp_cc(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &cnf->optional_start, cnf->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_cref(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_refused *ref = (struct sccp_connection_refused *)msg->l2h;
@@ -1039,6 +1084,7 @@ static struct xua_msg *sccp_to_xua_cref(struct msgb *msg, struct xua_msg *xua)
 	return sccp_to_xua_opt(msg, &ref->optional_start, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static int sua_to_sccp_cref(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_refused *ref;
@@ -1052,6 +1098,7 @@ static int sua_to_sccp_cref(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &ref->optional_start, ref->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_rlsd(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_released *rlsd = (struct sccp_connection_released *)msg->l2h;
@@ -1079,6 +1126,7 @@ static int sua_to_sccp_rlsd(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &rlsd->optional_start, rlsd->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_rlc(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_release_complete *rlc;
@@ -1102,6 +1150,7 @@ static int sua_to_sccp_rlc(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_dt1(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_form1 *dt1 = (struct sccp_data_form1 *) msg->l2h;
@@ -1130,6 +1179,7 @@ static int sua_to_sccp_dt1(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_udt(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_unitdata *udt = (struct sccp_data_unitdata *)msg->l2h;
@@ -1150,9 +1200,16 @@ static struct xua_msg *sccp_to_xua_udt(struct msgb *msg, struct xua_msg *xua)
 
 }
 
+static int sua_to_sccp_xudt(struct msgb *msg, struct xua_msg *xua);
+
 static int sua_to_sccp_udt(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_unitdata *udt;
+
+	/* Use XUDT if we have a hop counter on the SUA side */
+	if (xua_msg_find_tag(xua, SUA_IEI_S7_HOP_CTR))
+		return sua_to_sccp_xudt(msg, xua);
+
 	udt = (struct sccp_data_unitdata *) msgb_put(msg, sizeof(*udt));
 
 	/* Fixed Part */
@@ -1165,6 +1222,47 @@ static int sua_to_sccp_udt(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
+static struct xua_msg *sccp_to_xua_xudt(struct msgb *msg, struct xua_msg *xua)
+{
+	struct sccp_data_ext_unitdata *xudt = (struct sccp_data_ext_unitdata *)msg->l2h;
+
+	/* Fixed Part */
+	xua_msg_add_u32(xua, SUA_IEI_PROTO_CLASS, xudt->proto_class);
+	xua_msg_add_u32(xua, SUA_IEI_S7_HOP_CTR, xudt->hop_counter);
+	/* Variable Part */
+	if (!sccp_ptr_part_consistent(msg, &xudt->variable_called))
+		return NULL;
+	sccp_addr_to_sua_ptr(xua, SUA_IEI_DEST_ADDR, msg, &xudt->variable_called);
+	if (!sccp_ptr_part_consistent(msg, &xudt->variable_calling))
+		return NULL;
+	sccp_addr_to_sua_ptr(xua, SUA_IEI_SRC_ADDR, msg, &xudt->variable_calling);
+	if (!sccp_ptr_part_consistent(msg, &xudt->variable_data))
+		return NULL;
+	sccp_data_to_sua_ptr(xua, SUA_IEI_DATA, msg, &xudt->variable_data);
+	/* Optional Part */
+	return sccp_to_xua_opt(msg, &xudt->optional_start, xua);
+
+}
+
+static int sua_to_sccp_xudt(struct msgb *msg, struct xua_msg *xua)
+{
+	struct sccp_data_ext_unitdata *xudt;
+	xudt = (struct sccp_data_ext_unitdata *) msgb_put(msg, sizeof(*xudt));
+
+	/* Fixed Part */
+	xudt->type = SCCP_MSG_TYPE_XUDT;
+	xudt->proto_class = xua_msg_get_u32(xua, SUA_IEI_PROTO_CLASS);
+	xudt->hop_counter = xua_msg_get_u32(xua, SUA_IEI_S7_HOP_CTR);
+	/* Variable Part */
+	sccp_add_var_addr(msg, &xudt->variable_called, xua, SUA_IEI_DEST_ADDR);
+	sccp_add_var_addr(msg, &xudt->variable_calling, xua, SUA_IEI_SRC_ADDR);
+	sccp_add_variable_part(msg, &xudt->variable_data, xua, SUA_IEI_DATA);
+	/* Optional Part */
+	return xua_ies_to_sccp_opts(msg, &xudt->optional_start, xudt->type, xua);
+}
+
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_udts(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_unitdata_service *udts;
@@ -1186,9 +1284,16 @@ static struct xua_msg *sccp_to_xua_udts(struct msgb *msg, struct xua_msg *xua)
 
 }
 
+static int sua_to_sccp_xudts(struct msgb *msg, struct xua_msg *xua);
+
 static int sua_to_sccp_udts(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_unitdata_service *udts;
+
+	/* Use XUDTS if we have a hop counter */
+	if (xua_msg_find_tag(xua, SUA_IEI_S7_HOP_CTR))
+		return sua_to_sccp_xudts(msg, xua);
+
 	udts = (struct sccp_data_unitdata_service *) msgb_put(msg, sizeof(*udts));
 
 	/* Fixed Part */
@@ -1201,6 +1306,47 @@ static int sua_to_sccp_udts(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
+static struct xua_msg *sccp_to_xua_xudts(struct msgb *msg, struct xua_msg *xua)
+{
+	struct sccp_data_ext_unitdata_service *xudts;
+	xudts = (struct sccp_data_ext_unitdata_service *)msg->l2h;
+
+	/* Fixed Part */
+	xua_msg_add_u32(xua, SUA_IEI_CAUSE, SUA_CAUSE_T_RETURN | xudts->return_cause);
+	xua_msg_add_u32(xua, SUA_IEI_S7_HOP_CTR, xudts->hop_counter);
+	/* Variable Part */
+	if (!sccp_ptr_part_consistent(msg, &xudts->variable_called))
+		return NULL;
+	sccp_addr_to_sua_ptr(xua, SUA_IEI_DEST_ADDR, msg, &xudts->variable_called);
+	if (!sccp_ptr_part_consistent(msg, &xudts->variable_calling))
+		return NULL;
+	sccp_addr_to_sua_ptr(xua, SUA_IEI_SRC_ADDR, msg, &xudts->variable_calling);
+	if (!sccp_ptr_part_consistent(msg, &xudts->variable_data))
+		return NULL;
+	sccp_data_to_sua_ptr(xua, SUA_IEI_DATA, msg, &xudts->variable_data);
+	/* Optional Part */
+	return sccp_to_xua_opt(msg, &xudts->optional_start, xua);
+}
+
+static int sua_to_sccp_xudts(struct msgb *msg, struct xua_msg *xua)
+{
+	struct sccp_data_ext_unitdata_service *xudts;
+	xudts = (struct sccp_data_ext_unitdata_service *) msgb_put(msg, sizeof(*xudts));
+
+	/* Fixed Part */
+	xudts->type = SCCP_MSG_TYPE_XUDTS;
+	xudts->return_cause = xua_msg_get_u32(xua, SUA_IEI_CAUSE) & 0xff;
+	xudts->hop_counter = xua_msg_get_u32(xua, SUA_IEI_S7_HOP_CTR);
+	/* Variable Part */
+	sccp_add_var_addr(msg, &xudts->variable_called, xua, SUA_IEI_DEST_ADDR);
+	sccp_add_var_addr(msg, &xudts->variable_calling, xua, SUA_IEI_SRC_ADDR);
+	sccp_add_variable_part(msg, &xudts->variable_data, xua, SUA_IEI_DATA);
+	/* Optional Part */
+	return xua_ies_to_sccp_opts(msg, &xudts->optional_start, xudts->type, xua);
+}
+
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_it(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_it *it = (struct sccp_data_it *)msg->l2h;
@@ -1234,6 +1380,7 @@ static int sua_to_sccp_it(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_err(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_proto_err *err = (struct sccp_proto_err *)msg->l2h;
@@ -1275,34 +1422,64 @@ struct xua_msg *osmo_sccp_to_xua(struct msgb *msg)
 	switch (msg->l2h[0]) {
 	case SCCP_MSG_TYPE_CR:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_CORE);
-		return sccp_to_xua_cr(msg, xua);
+		if (!sccp_to_xua_cr(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_CC:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COAK);
-		return sccp_to_xua_cc(msg, xua);
+		if (!sccp_to_xua_cc(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_CREF:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COREF);
-		return sccp_to_xua_cref(msg, xua);
+		if (!sccp_to_xua_cref(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_RLSD:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_RELRE);
-		return sccp_to_xua_rlsd(msg, xua);
+		if (!sccp_to_xua_rlsd(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_RLC:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_RELCO);
-		return sccp_to_xua_rlc(msg, xua);
+		if (!sccp_to_xua_rlc(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_DT1:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_CODT);
-		return sccp_to_xua_dt1(msg, xua);
+		if (!sccp_to_xua_dt1(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_UDT:
 		xua->hdr = XUA_HDR(SUA_MSGC_CL, SUA_CL_CLDT);
-		return sccp_to_xua_udt(msg, xua);
+		if (!sccp_to_xua_udt(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_UDTS:
 		xua->hdr = XUA_HDR(SUA_MSGC_CL, SUA_CL_CLDR);
-		return sccp_to_xua_udts(msg, xua);
+		if (!sccp_to_xua_udts(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_IT:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COIT);
-		return sccp_to_xua_it(msg, xua);
+		if (!sccp_to_xua_it(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_ERR:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COERR);
-		return sccp_to_xua_err(msg, xua);
+		if (!sccp_to_xua_err(msg, xua))
+			goto malformed;
+		return xua;
+	case SCCP_MSG_TYPE_XUDT:
+		xua->hdr = XUA_HDR(SUA_MSGC_CL, SUA_CL_CLDT);
+		if (!sccp_to_xua_xudt(msg, xua))
+			goto malformed;
+		return xua;
+	case SCCP_MSG_TYPE_XUDTS:
+		xua->hdr = XUA_HDR(SUA_MSGC_CL, SUA_CL_CLDR);
+		if (!sccp_to_xua_xudts(msg, xua))
+			goto malformed;
+		return xua;
 	/* Unsupported Message Types */
 	case SCCP_MSG_TYPE_DT2:
 	case SCCP_MSG_TYPE_AK:
@@ -1310,10 +1487,12 @@ struct xua_msg *osmo_sccp_to_xua(struct msgb *msg)
 	case SCCP_MSG_TYPE_EA:
 	case SCCP_MSG_TYPE_RSR:
 	case SCCP_MSG_TYPE_RSC:
-	case SCCP_MSG_TYPE_XUDT:
-	case SCCP_MSG_TYPE_XUDTS:
 	case SCCP_MSG_TYPE_LUDT:
 	case SCCP_MSG_TYPE_LUDTS:
+		LOGP(DLSUA, LOGL_ERROR, "Unsupported SCCP message %s\n",
+			osmo_sccp_msg_type_name(msg->l2h[0]));
+		xua_msg_free(xua);
+		return NULL;
 	default:
 		LOGP(DLSUA, LOGL_ERROR, "Unsupported SCCP message type %u\n",
 			msg->l2h[0]);
@@ -1322,6 +1501,12 @@ struct xua_msg *osmo_sccp_to_xua(struct msgb *msg)
 	}
 
 	return NULL;
+
+malformed:
+	LOGP(DLSUA, LOGL_ERROR, "Malformed SCCP message %s\n",
+	     osmo_sccp_msg_type_name(msg->l2h[0]));
+	xua_msg_free(xua);
+	return NULL;
 }
 
 /*! \brief convert parsed SUA message to SCCP message
@@ -1385,8 +1570,11 @@ struct msgb *osmo_sua_to_sccp(struct xua_msg *xua)
 		goto out_err;
 	}
 
-	if (rc < 0)
+	if (rc < 0)  {
+		LOGP(DLSUA, LOGL_ERROR, "Malformed SUA message %s\n",
+			xua_hdr_dump(xua, &xua_dialect_sua));
 		goto out_err;
+	}
 
 	return msg;
 
