@@ -1,8 +1,10 @@
 /* Minimal implementation of RFC 3868 - SCCP User Adaptation Layer */
 
-/* (C) 2015-2017 by Harald Welte <laforge@gnumonks.org>
+/* (C) 2015-2021 by Harald Welte <laforge@gnumonks.org>
  * All Rights Reserved
  *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -44,6 +46,7 @@
 #include "xua_asp_fsm.h"
 #include "xua_internal.h"
 #include "sccp_internal.h"
+#include "ss7_internal.h"
 
 /* Appendix C.4 of Q.714 (all in milliseconds) */
 #define CONNECTION_TIMER	( 1 * 60 * 100)
@@ -60,10 +63,15 @@
 
 struct msgb *sccp_msgb_alloc(const char *name)
 {
+	struct msgb *msg;
 	if (!name)
 		name = "SCCP";
-	return msgb_alloc_headroom(SCCP_MSG_SIZE+SCCP_MSG_HEADROOM,
-				   SCCP_MSG_HEADROOM, name);
+	msg = msgb_alloc_headroom(SCCP_MSG_SIZE+SCCP_MSG_HEADROOM,
+				  SCCP_MSG_HEADROOM, name);
+	if (!msg)
+		return NULL;
+	msg->l2h = msg->tail;
+	return msg;
 }
 
 /***********************************************************************
@@ -460,6 +468,13 @@ int sua_addr_parse_part(struct osmo_sccp_addr *out,
 			out->ip.v4.s_addr = *p32;
 			out->presence |= OSMO_SCCP_ADDR_T_IPv4;
 			break;
+		case SUA_IEI_IPv6:
+			if (par_datalen != 16)
+				goto subpar_fail;
+			/* no endian conversion, both network order */
+			memcpy(&out->ip.v6, par->data, 16);
+			out->presence |= OSMO_SCCP_ADDR_T_IPv6;
+			break;
 		default:
 			LOGP(DLSUA, LOGL_ERROR, "SUA IEI 0x%04x: Unknown subpart tag %hd\n",
 			     param->tag, par_tag);
@@ -496,10 +511,19 @@ int sua_addr_parse(struct osmo_sccp_addr *out, struct xua_msg *xua, uint16_t iei
 /* connectionless messages received from socket */
 static int sua_rx_cl(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 {
+	struct xua_msg_part *rctx_ie = xua_msg_find_tag(xua, SUA_IEI_ROUTE_CTX);
 	struct osmo_sccp_instance *inst = asp->inst->sccp;
+	struct osmo_ss7_as *as;
+	int rc;
 
 	OSMO_ASSERT(xua->hdr.msg_class == SUA_MSGC_CL);
 
+	rc = xua_find_as_for_asp(&as, asp, rctx_ie);
+	if (rc)
+		return rc;
+
+	rate_ctr_inc2(as->ctrg, SS7_AS_CTR_RX_MSU_TOTAL);
+
 	switch (xua->hdr.msg_type) {
 	case 0: /* Reserved, permitted by ETSI TS 101 592 5.2.3.2 */
 	case SUA_CL_CLDT:
@@ -517,10 +541,19 @@ static int sua_rx_cl(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 /* connection-oriented messages received from socket */
 static int sua_rx_co(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 {
+	struct xua_msg_part *rctx_ie = xua_msg_find_tag(xua, SUA_IEI_ROUTE_CTX);
 	struct osmo_sccp_instance *inst = asp->inst->sccp;
+	struct osmo_ss7_as *as;
+	int rc;
 
 	OSMO_ASSERT(xua->hdr.msg_class == SUA_MSGC_CO);
 
+	rc = xua_find_as_for_asp(&as, asp, rctx_ie);
+	if (rc)
+		return rc;
+
+	rate_ctr_inc2(as->ctrg, SS7_AS_CTR_RX_MSU_TOTAL);
+
 	switch (xua->hdr.msg_type) {
 	case 0: /* Reserved, permitted by ETSI TS 101 592 5.2.3.2 */
 	case SUA_CO_CORE:
@@ -633,6 +666,8 @@ static int sua_rx_asp(struct osmo_ss7_asp *asp, struct xua_msg *xua)
 	return 0;
 }
 
+static int sua_rx_snm(struct osmo_ss7_asp *asp, struct xua_msg *xua);
+
 /*! \brief process SUA message received from socket
  *  \param[in] asp Application Server Process receiving \ref msg
  *  \param[in] msg received message buffer
@@ -708,6 +743,8 @@ int sua_rx_msg(struct osmo_ss7_asp *asp, struct msgb *msg)
 		rc = sua_rx_mgmt(asp, xua);
 		break;
 	case SUA_MSGC_SNM:
+		rc = sua_rx_snm(asp, xua);
+		break;
 	case SUA_MSGC_RKM:
 		/* FIXME */
 		LOGPASP(asp, DLSUA, LOGL_NOTICE, "Received unsupported SUA "
@@ -733,3 +770,235 @@ out:
 	return rc;
 }
 
+/***********************************************************************
+ * SSNM msg generation
+ ***********************************************************************/
+
+/* 3.4.1 Destination Unavailable (DUNA) */
+static struct xua_msg *sua_encode_duna(const uint32_t *rctx, unsigned int num_rctx,
+					const uint32_t *aff_pc, unsigned int num_aff_pc,
+					const uint32_t *ssn, const uint32_t *smi, const char *info_string)
+{
+	struct xua_msg *xua = xua_msg_alloc();
+
+	xua->hdr = XUA_HDR(SUA_MSGC_SNM, SUA_SNM_DUNA);
+	xua->hdr.version = SUA_VERSION;
+	if (rctx)
+		xua_msg_add_data(xua, SUA_IEI_ROUTE_CTX, num_rctx * 4, (const uint8_t *)rctx);
+
+	xua_msg_add_data(xua, SUA_IEI_AFFECTED_PC, num_aff_pc * 4, (const uint8_t *) aff_pc);
+
+	if (ssn)
+		xua_msg_add_u32(xua, SUA_IEI_SSN, *ssn);
+
+	if (smi)
+		xua_msg_add_u32(xua, SUA_IEI_SSN, *smi);
+
+	if (info_string) {
+		xua_msg_add_data(xua, SUA_IEI_INFO_STRING,
+				 strlen(info_string)+1,
+				 (const uint8_t *) info_string);
+	}
+	return xua;
+}
+
+/* 3.4.2 Destination Available (DAVA) */
+static struct xua_msg *sua_encode_dava(const uint32_t *rctx, unsigned int num_rctx,
+					const uint32_t *aff_pc, unsigned int num_aff_pc,
+					const uint32_t *ssn, const uint32_t *smi, const char *info_string)
+{
+	/* encoding is exactly identical to DUNA */
+	struct xua_msg *xua = sua_encode_duna(rctx, num_rctx, aff_pc, num_aff_pc, ssn, smi, info_string);
+	if (xua)
+		xua->hdr.msg_type = SUA_SNM_DAVA;
+	return xua;
+}
+
+#if 0 /* not used so far */
+/* 3.4.3 Destination Available (DAUD) */
+static struct xua_msg *sua_encode_daud(const uint32_t *rctx, unsigned int num_rctx,
+					const uint32_t *aff_pc, unsigned int num_aff_pc,
+					const uint32_t *ssn, const uint32_t *smi, const char *info_string)
+{
+	/* encoding is exactly identical to DUNA */
+	struct xua_msg *xua = sua_encode_duna(rctx, num_rctx, aff_pc, num_aff_pc, ssn, smi, info_string);
+	if (xua)
+		xua->hdr.msg_type = SUA_SNM_DAUD;
+	return xua;
+}
+#endif
+
+/* 3.4.5 Destination User Part Unavailable (DUPU) */
+static struct xua_msg *sua_encode_dupu(const uint32_t *rctx, unsigned int num_rctx,
+					uint32_t dpc, uint16_t user, uint16_t cause,
+					const char *info_string)
+{
+	struct xua_msg *xua = xua_msg_alloc();
+	uint32_t user_cause = (user << 16) | cause;
+
+	xua->hdr = XUA_HDR(SUA_MSGC_SNM, SUA_SNM_DUNA);
+	xua->hdr.version = SUA_VERSION;
+	if (rctx)
+		xua_msg_add_data(xua, SUA_IEI_ROUTE_CTX, num_rctx * sizeof(*rctx), (const uint8_t *)rctx);
+
+	xua_msg_add_u32(xua, SUA_IEI_AFFECTED_PC, dpc);
+	xua_msg_add_u32(xua, SUA_IEI_USER_CAUSE, user_cause);
+
+	if (info_string) {
+		xua_msg_add_data(xua, SUA_IEI_INFO_STRING, strlen(info_string)+1,
+				 (const uint8_t *) info_string);
+	}
+	return xua;
+}
+
+/*! Transmit SSNM DUNA/DAVA message indicating [un]availability of certain point code[s]
+ *  \param[in] asp ASP through whihc to transmit message. Must be ACTIVE.
+ *  \param[in] rctx array of Routing Contexts in network byte order.
+ *  \param[in] num_rctx number of rctx
+ *  \param[in] aff_pc array of 'Affected Point Code' in network byte order.
+ *  \param[in] num_aff_pc number of aff_pc
+ *  \param[in] aff_ssn affected SSN (optional)
+ *  \param[in] smi subsystem multiplicity indicator (optional)
+ *  \param[in] info_string optional information strng (can be NULL).
+ *  \param[in] available are aff_pc now available (true) or unavailable (false) */
+void sua_tx_snm_available(struct osmo_ss7_asp *asp, const uint32_t *rctx, unsigned int num_rctx,
+			  const uint32_t *aff_pc, unsigned int num_aff_pc, const uint32_t *aff_ssn,
+			  const uint32_t *smi, const char *info_string, bool available)
+{
+	struct xua_msg *xua;
+
+	if (available)
+		xua = sua_encode_dava(rctx, num_rctx, aff_pc, num_aff_pc, aff_ssn, smi, info_string);
+	else
+		xua = sua_encode_duna(rctx, num_rctx, aff_pc, num_aff_pc, aff_ssn, smi, info_string);
+
+	sua_tx_xua_asp(asp, xua);
+}
+
+
+/*! Transmit SSNM SCON message indicating congestion
+ *  \param[in] asp ASP through which to transmit message. Must be ACTIVE.
+ *  \param[in] rctx array of Routing Contexts in network byte order.
+ *  \param[in] num_rctx number of rctx
+ *  \param[in] aff_pc array of 'Affected Point Code' in network byte order.
+ *  \param[in] num_aff_pc number of aff_pc
+ *  \param[in] ssn optional SSN (can be NULL)
+ *  \param[in] cong_level optional congestion level (can be NULL)
+ *  \param[in] info_string optional information string (can be NULL). */
+void sua_tx_snm_congestion(struct osmo_ss7_asp *asp, const uint32_t *rctx, unsigned int num_rctx,
+			   const uint32_t *aff_pc, unsigned int num_aff_pc, const uint32_t *ssn,
+			   const uint8_t cong_level, const char *info_string)
+{
+	struct xua_msg *xua = xua_msg_alloc();
+
+	xua->hdr = XUA_HDR(SUA_MSGC_SNM, SUA_SNM_SCON);
+	xua->hdr.version = SUA_VERSION;
+	if (rctx)
+		xua_msg_add_data(xua, SUA_IEI_ROUTE_CTX, num_rctx * sizeof(*rctx), (const uint8_t *)rctx);
+	xua_msg_add_data(xua, SUA_IEI_AFFECTED_PC, num_aff_pc * sizeof(*aff_pc), (const uint8_t *) aff_pc);
+	if (ssn)
+		xua_msg_add_u32(xua, SUA_IEI_SSN, *ssn);
+	xua_msg_add_u32(xua, SUA_IEI_CONG_LEVEL, cong_level);
+	if (info_string)
+		xua_msg_add_data(xua, SUA_IEI_INFO_STRING, strlen(info_string)+1, (const uint8_t *) info_string);
+}
+
+
+/*! Transmit SSNM DUPU message indicating user unavailability.
+ *  \param[in] asp ASP through which to transmit message. Must be ACTIVE.
+ *  \param[in] rctx array of Routing Contexts in network byte order.
+ *  \param[in] num_rctx number of rctx
+ *  \param[in] dpc affected point code
+ *  \param[in] user the user (SI) that is unavailable
+ *  \param[in] cause the cause of the user unavailability
+ *  \param[in] info_string optional information string (can be NULL). */
+void sua_tx_dupu(struct osmo_ss7_asp *asp, const uint32_t *rctx, unsigned int num_rctx,
+		 uint32_t dpc, uint16_t user, uint16_t cause, const char *info_str)
+{
+	struct xua_msg *xua = sua_encode_dupu(rctx, num_rctx, dpc, user, cause, info_str);
+	sua_tx_xua_asp(asp, xua);
+}
+
+/* received SNM message on ASP side */
+static int sua_rx_snm_asp(struct osmo_ss7_asp *asp, struct xua_msg *xua)
+{
+	struct osmo_ss7_as *as = NULL;
+	struct xua_msg_part *rctx_ie = xua_msg_find_tag(xua, SUA_IEI_ROUTE_CTX);
+	int rc;
+
+	rc = xua_find_as_for_asp(&as, asp, rctx_ie);
+	if (rc)
+		return rc;
+
+	switch (xua->hdr.msg_type) {
+	case SUA_SNM_DUNA:
+		xua_snm_rx_duna(asp, as, xua);
+		break;
+	case SUA_SNM_DAVA:
+		xua_snm_rx_dava(asp, as, xua);
+		break;
+	case SUA_SNM_DUPU:
+		xua_snm_rx_dupu(asp, as, xua);
+		break;
+	case SUA_SNM_SCON:
+	case SUA_SNM_DRST:
+		LOGPASP(asp, DLSUA, LOGL_NOTICE, "Received unsupported SUA SNM message type %u\n",
+			xua->hdr.msg_type);
+		/* silently ignore those to not confuse the sender */
+		break;
+	case SUA_SNM_DAUD:
+		/* RFC states only permitted in ASP->SG direction, not reverse. But some
+		 * equipment still sends it to us as ASP ?!? */
+		if (asp->cfg.quirks & OSMO_SS7_ASP_QUIRK_DAUD_IN_ASP) {
+			LOGPASP(asp, DLSUA, LOGL_NOTICE, "quirk daud_in_asp active: Accepting DAUD "
+				"despite being in ASP role\n");
+			xua_snm_rx_daud(asp, xua);
+		} else {
+			LOGPASP(asp, DLSUA, LOGL_ERROR, "DAUD not permitted in ASP role\n");
+			return SUA_ERR_UNSUPP_MSG_TYPE;
+		}
+		break;
+	default:
+		return SUA_ERR_UNSUPP_MSG_TYPE;
+	}
+
+	return 0;
+}
+
+/* received SNM message on SG side */
+static int sua_rx_snm_sg(struct osmo_ss7_asp *asp, struct xua_msg *xua)
+{
+	switch (xua->hdr.msg_type) {
+	case SUA_SNM_DAUD:	/* Audit: ASP inquires about availability of Point Codes */
+		xua_snm_rx_daud(asp, xua);
+		break;
+	default:
+		return SUA_ERR_UNSUPP_MSG_TYPE;
+	}
+
+	return 0;
+}
+
+static int sua_rx_snm(struct osmo_ss7_asp *asp, struct xua_msg *xua)
+{
+	/* SNM only permitted in ACTIVE state */
+	if (asp->fi->state != XUA_ASP_S_ACTIVE) {
+		if (asp->fi->state == XUA_ASP_S_INACTIVE && asp->cfg.quirks & OSMO_SS7_ASP_QUIRK_SNM_INACTIVE) {
+			LOGPASP(asp, DLSUA, LOGL_NOTICE, "quirk snm_inactive active: "
+				"Accepting SSNM in state %s\n", osmo_fsm_inst_state_name(asp->fi));
+		} else {
+			LOGPASP(asp, DLM3UA, LOGL_ERROR, "Rx SUA SSNM not permitted "
+				"while ASP in state %s\n", osmo_fsm_inst_state_name(asp->fi));
+			return SUA_ERR_UNEXPECTED_MSG;
+		}
+	}
+
+	switch (asp->cfg.role) {
+	case OSMO_SS7_ASP_ROLE_SG:
+		return sua_rx_snm_sg(asp, xua);
+	case OSMO_SS7_ASP_ROLE_ASP:
+		return sua_rx_snm_asp(asp, xua);
+	default:
+		return SUA_ERR_UNSUPP_MSG_CLASS;
+	}
+}
