@@ -59,7 +59,7 @@ void xua_msg_free(struct xua_msg *msg)
 }
 
 int xua_msg_add_data(struct xua_msg *msg, uint16_t tag,
-		      uint16_t len, uint8_t *dat)
+		      uint16_t len, const uint8_t *dat)
 {
 	struct xua_msg_part *part;
 
@@ -283,13 +283,13 @@ int xua_msg_add_u32(struct xua_msg *xua, uint16_t iei, uint32_t val)
 	return xua_msg_add_data(xua, iei, sizeof(val_n), (uint8_t *) &val_n);
 }
 
-uint32_t xua_msg_part_get_u32(struct xua_msg_part *part)
+uint32_t xua_msg_part_get_u32(const struct xua_msg_part *part)
 {
 	OSMO_ASSERT(part->len >= 4);
 	return ntohl(*(uint32_t *)part->dat);
 }
 
-uint32_t xua_msg_get_u32(struct xua_msg *xua, uint16_t iei)
+uint32_t xua_msg_get_u32(const struct xua_msg *xua, uint16_t iei)
 {
 	struct xua_msg_part *part = xua_msg_find_tag(xua, iei);
 	if (!part)
@@ -297,6 +297,39 @@ uint32_t xua_msg_get_u32(struct xua_msg *xua, uint16_t iei)
 	return xua_msg_part_get_u32(part);
 }
 
+/* get a U32 IE in host-byte-order, returning whether it exists (!= NULL) or not (== NULL) */
+const uint32_t *xua_msg_get_u32p(const struct xua_msg *xua, uint16_t iei, uint32_t *out)
+{
+	struct xua_msg_part *part = xua_msg_find_tag(xua, iei);
+	if (!part)
+		return NULL;
+	*out = xua_msg_part_get_u32(part);
+	return out;
+}
+
+const char *xua_msg_part_get_str(const struct xua_msg_part *part)
+{
+	static char __thread buf[256];
+
+	if (part->len == 0)
+		return "";
+	/* RFC3868 3.9.4: Length of the INFO String parameter is from 0 to 255 octets */
+	if (part->len > 255)
+		return "<invalid-string-len>";
+
+	memcpy(buf, part->dat, part->len);
+	buf[part->len] = '\0';
+	return buf;
+}
+
+const char *xua_msg_get_str(const struct xua_msg *xua, uint16_t iei)
+{
+	struct xua_msg_part *part = xua_msg_find_tag(xua, iei);
+	if (!part)
+		return NULL;
+	return xua_msg_part_get_str(part);
+}
+
 void xua_part_add_gt(struct msgb *msg, const struct osmo_sccp_gt *gt)
 {
 	uint16_t *len_ptr;
@@ -376,9 +409,9 @@ int xua_msg_add_sccp_addr(struct xua_msg *xua, uint16_t iei, const struct osmo_s
 		msgb_t16l16vp_put_u32(tmp, SUA_IEI_SSN, addr->ssn);
 	}
 	if (addr->presence & OSMO_SCCP_ADDR_T_IPv4) {
-		msgb_t16l16vp_put_u32(tmp, SUA_IEI_IPv4, ntohl(addr->ip.v4.s_addr));
+		msgb_t16l16vp_put(tmp, SUA_IEI_IPv4, sizeof(addr->ip.v4), (const uint8_t *)&addr->ip.v4);
 	} else if (addr->presence & OSMO_SCCP_ADDR_T_IPv6) {
-		/* FIXME: IPv6 address */
+		msgb_t16l16vp_put(tmp, SUA_IEI_IPv6, sizeof(addr->ip.v6), (const uint8_t *)&addr->ip.v6);
 	}
 	rc = xua_msg_add_data(xua, iei, msgb_length(tmp), tmp->data);
 	msgb_free(tmp);
